"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AccountType(str, Enum):
    r"""The type of bank account e.g. checking, savings, loan, creditCard, prepaidCard."""

    CHECKING = "checking"
    SAVINGS = "savings"
    LOAN = "loan"
    CREDIT_CARD = "creditCard"
    PREPAID_CARD = "prepaidCard"
