"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import importlib.metadata

__title__: str = "codat-bankfeeds"
__version__: str = "11.0.0"
__openapi_doc_version__: str = "3.0.0"
__gen_version__: str = "2.497.0"
__user_agent__: str = "speakeasy-sdk/python 11.0.0 2.497.0 3.0.0 codat-bankfeeds"

try:
    if __package__ is not None:
        __version__ = importlib.metadata.version(__package__)
except importlib.metadata.PackageNotFoundError:
    pass
