"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from codat_bankfeeds.models.shared import (
    dataconnectionstatus as shared_dataconnectionstatus,
)
from codat_bankfeeds.types import BaseModel
from codat_bankfeeds.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UnlinkConnectionUpdateConnectionTypedDict(TypedDict):
    status: NotRequired[shared_dataconnectionstatus.DataConnectionStatus]
    r"""The current authorization status of the data connection."""


class UnlinkConnectionUpdateConnection(BaseModel):
    status: Optional[shared_dataconnectionstatus.DataConnectionStatus] = None
    r"""The current authorization status of the data connection."""


class UnlinkConnectionRequestTypedDict(TypedDict):
    company_id: str
    r"""Unique identifier for a company."""
    connection_id: str
    r"""Unique identifier for a connection."""
    request_body: NotRequired[UnlinkConnectionUpdateConnectionTypedDict]


class UnlinkConnectionRequest(BaseModel):
    company_id: Annotated[
        str,
        pydantic.Field(alias="companyId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Unique identifier for a company."""

    connection_id: Annotated[
        str,
        pydantic.Field(alias="connectionId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Unique identifier for a connection."""

    request_body: Annotated[
        Optional[UnlinkConnectionUpdateConnection],
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ] = None
