"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .configurationschedule import ConfigurationSchedule, ConfigurationScheduleTypedDict
from .syncconfiguration import SyncConfiguration, SyncConfigurationTypedDict
from codat_bankfeeds.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ConfigurationTypedDict(TypedDict):
    accounting_software_company_name: NotRequired[str]
    r"""The company name defined in the accounting software."""
    company_id: NotRequired[str]
    r"""Unique identifier for your SMB in Codat."""
    configuration: NotRequired[SyncConfigurationTypedDict]
    configured: NotRequired[bool]
    r"""True if the company has been configured."""
    enabled: NotRequired[bool]
    r"""Enabled or disable bank feeds."""
    schedule: NotRequired[ConfigurationScheduleTypedDict]


class Configuration(BaseModel):
    accounting_software_company_name: Annotated[
        Optional[str], pydantic.Field(alias="accountingSoftwareCompanyName")
    ] = None
    r"""The company name defined in the accounting software."""

    company_id: Annotated[Optional[str], pydantic.Field(alias="companyId")] = None
    r"""Unique identifier for your SMB in Codat."""

    configuration: Optional[SyncConfiguration] = None

    configured: Optional[bool] = None
    r"""True if the company has been configured."""

    enabled: Optional[bool] = None
    r"""Enabled or disable bank feeds."""

    schedule: Optional[ConfigurationSchedule] = None
