"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from codat_bankfeeds.models.shared import configuration as shared_configuration
from codat_bankfeeds.types import BaseModel
from codat_bankfeeds.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SetConfigurationRequestTypedDict(TypedDict):
    company_id: str
    r"""Unique identifier for a company."""
    configuration: NotRequired[shared_configuration.ConfigurationTypedDict]


class SetConfigurationRequest(BaseModel):
    company_id: Annotated[
        str,
        pydantic.Field(alias="companyId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Unique identifier for a company."""

    configuration: Annotated[
        Optional[shared_configuration.Configuration],
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ] = None
