"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .datatype import DataType
from codat_bankfeeds.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PushOperationRefTypedDict(TypedDict):
    data_type: NotRequired[DataType]
    r"""Available data types"""
    id: NotRequired[str]
    r"""Unique identifier for a push operation."""


class PushOperationRef(BaseModel):
    data_type: Annotated[Optional[DataType], pydantic.Field(alias="dataType")] = None
    r"""Available data types"""

    id: Optional[str] = None
    r"""Unique identifier for a push operation."""
