"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from codat_bankfeeds.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ConfigurationContactRefTypedDict(TypedDict):
    id: NotRequired[str]
    r"""Unique identifier for the supplier/customer."""
    name: NotRequired[str]
    r"""The supplier/customer's name."""


class ConfigurationContactRef(BaseModel):
    id: Optional[str] = None
    r"""Unique identifier for the supplier/customer."""

    name: Optional[str] = None
    r"""The supplier/customer's name."""
