"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .halref import HalRef, HalRefTypedDict
from codat_bankfeeds.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LinksTypedDict(TypedDict):
    current: HalRefTypedDict
    self_: HalRefTypedDict
    next: NotRequired[HalRefTypedDict]
    previous: NotRequired[HalRefTypedDict]


class Links(BaseModel):
    current: HalRef

    self_: Annotated[HalRef, pydantic.Field(alias="self")]

    next: Optional[HalRef] = None

    previous: Optional[HalRef] = None
