"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bankaccountoption import BankAccountOption, BankAccountOptionTypedDict
from codat_bankfeeds.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SyncAsBankFeedsTypedDict(TypedDict):
    bank_account_options: NotRequired[List[BankAccountOptionTypedDict]]
    enable_sync: NotRequired[bool]
    r"""True if bank feeds sync is enabled."""
    selected_bank_account_id: NotRequired[str]
    r"""The bank account ID being synced."""


class SyncAsBankFeeds(BaseModel):
    bank_account_options: Annotated[
        Optional[List[BankAccountOption]], pydantic.Field(alias="bankAccountOptions")
    ] = None

    enable_sync: Annotated[Optional[bool], pydantic.Field(alias="enableSync")] = None
    r"""True if bank feeds sync is enabled."""

    selected_bank_account_id: Annotated[
        Optional[str], pydantic.Field(alias="selectedBankAccountId")
    ] = None
    r"""The bank account ID being synced."""
