"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from codat_bankfeeds.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class BankAccountOptionTypedDict(TypedDict):
    account_type: NotRequired[str]
    r"""The type of account."""
    id: NotRequired[str]
    r"""Unique identifier for the bank account."""
    name: NotRequired[str]
    r"""Name of bank account."""


class BankAccountOption(BaseModel):
    account_type: Annotated[Optional[str], pydantic.Field(alias="accountType")] = None
    r"""The type of account."""

    id: Optional[str] = None
    r"""Unique identifier for the bank account."""

    name: Optional[str] = None
    r"""Name of bank account."""
